#include "mode/install_zerotwoxci_mode.hpp"
extern "C" {
  #include "xci2.h"
}
#include <cstring>
#include "install/install_nsp.hpp"
#include "nx/fs.hpp"
#include "ui/console_options_view.hpp"
#include "util/file_util.hpp"
#include "error.hpp"

namespace tin::ui
{
    InstallZeroTwoXCIMode::InstallZeroTwoXCIMode() :
        IMode("Install XCI")
    {

    }

    void InstallZeroTwoXCIMode::OnSelected()
    {
        tin::ui::ViewManager& manager = tin::ui::ViewManager::Instance();
        auto view = std::make_unique<tin::ui::ConsoleOptionsView>();
        view->AddEntry(m_name, tin::ui::ConsoleEntrySelectType::HEADING, nullptr);
        view->AddEntry("", tin::ui::ConsoleEntrySelectType::NONE, nullptr);

        nx::fs::IFileSystem fileSystem;
        fileSystem.OpenSdFileSystem();
        /*nx::fs::IDirectory dir = fileSystem.OpenDirectory("/tinfoil/zerotwoxci", FS_DIROPEN_DIRECTORY);
        u64 entryCount = dir.GetEntryCount();

        if (entryCount > 0)
        {
            auto dirEntries = std::make_unique<FsDirectoryEntry[]>(entryCount);
            dir.Read(0, dirEntries.get(), entryCount);

            for (unsigned int i = 0; i < entryCount; i++)
            {
                FsDirectoryEntry dirEntry = dirEntries[i];

                if (dirEntry.type != ENTRYTYPE_DIR)
                    continue;
                std::string ncasstringidk ("ncas");
                if ((ncasstringidk.compare(dirEntry.name) != 0)) view->AddEntry(dirEntry.name, ConsoleEntrySelectType::SELECT, std::bind(&InstallZeroTwoXCIMode::OnExtractedNSPSelected, this));
            }
        }*/
        auto xciList = tin::util::GetXCIList();

        if (xciList.size() > 0)
        {

            for (unsigned int i = 0; i < xciList.size(); i++)
            {
              view->AddEntry(xciList[i], ConsoleEntrySelectType::SELECT, std::bind(&InstallZeroTwoXCIMode::OnExtractedNSPSelected, this));
            }
        }

        manager.PushView(std::move(view));
    }

    void InstallZeroTwoXCIMode::OnExtractedNSPSelected()
    {
        // Retrieve previous selection
        tin::ui::ViewManager& manager = tin::ui::ViewManager::Instance();
        ConsoleOptionsView* prevView;

        if (!(prevView = dynamic_cast<ConsoleOptionsView*>(manager.GetCurrentView())))
        {
            throw std::runtime_error("Previous view must be a ConsoleOptionsView!");
        }

        m_name = prevView->GetSelectedOptionValue()->GetText();

        // Prepare the next view
        auto view = std::make_unique<tin::ui::ConsoleOptionsView>();
        view->AddEntry("Select Destination", tin::ui::ConsoleEntrySelectType::HEADING, nullptr);
        view->AddEntry("", tin::ui::ConsoleEntrySelectType::NONE, nullptr);
        view->AddEntry("SD Card", tin::ui::ConsoleEntrySelectType::SELECT, std::bind(&InstallZeroTwoXCIMode::OnDestinationSelected, this));
        view->AddEntry("NAND", tin::ui::ConsoleEntrySelectType::SELECT, std::bind(&InstallZeroTwoXCIMode::OnDestinationSelected, this));
        manager.PushView(std::move(view));
    }

    void InstallZeroTwoXCIMode::OnDestinationSelected()
    {
        // Retrieve previous selection
        tin::ui::ViewManager& manager = tin::ui::ViewManager::Instance();
        ConsoleOptionsView* prevView;

        if (!(prevView = dynamic_cast<ConsoleOptionsView*>(manager.GetCurrentView())))
        {
            throw std::runtime_error("Previous view must be a ConsoleOptionsView!");
        }

        auto destStr = prevView->GetSelectedOptionValue()->GetText();
        m_destStorageId = FsStorageId_SdCard;

        if (destStr == "NAND")
        {
            m_destStorageId = FsStorageId_NandUser;
        }

        auto view = std::make_unique<tin::ui::ConsoleOptionsView>();
        view->AddEntry("Ignore Required Firmware Version", tin::ui::ConsoleEntrySelectType::HEADING, nullptr);
        view->AddEntry("", tin::ui::ConsoleEntrySelectType::NONE, nullptr);
        view->AddEntry("No", tin::ui::ConsoleEntrySelectType::SELECT, std::bind(&InstallZeroTwoXCIMode::OnIgnoreReqFirmVersionSelected, this));
        view->AddEntry("Yes", tin::ui::ConsoleEntrySelectType::SELECT, std::bind(&InstallZeroTwoXCIMode::OnIgnoreReqFirmVersionSelected, this));
        manager.PushView(std::move(view));
    }

    void InstallZeroTwoXCIMode::OnIgnoreReqFirmVersionSelected()
    {
        // Retrieve previous selection
        tin::ui::ViewManager& manager = tin::ui::ViewManager::Instance();
        ConsoleOptionsView* prevView;

        if (!(prevView = dynamic_cast<ConsoleOptionsView*>(manager.GetCurrentView())))
        {
            throw std::runtime_error("Previous view must be a ConsoleOptionsView!");
        }

        auto optStr = prevView->GetSelectedOptionValue()->GetText();
        m_ignoreReqFirmVersion = false;

        if (optStr == "Yes")
        {
            m_ignoreReqFirmVersion = true;
        }

        char * pathchar = (char*)malloc(strlen("/tinfoil/zerotwoxci/")+strlen(m_name.c_str())+2);
        snprintf(pathchar, strlen("/tinfoil/zerotwoxci//")+strlen(m_name.c_str())+2, "/tinfoil/zerotwoxci/%s/", m_name.c_str());
        int reeeeeinit = reinitdir();
        int decompilexci = startxcidecompile(pathchar);
        if (decompilexci == 1) {
          printf("Failed to decompile XCI, maybe a corrupt dump? If dump isn't corrupt open an issue with the title of the game in the issue on ZeroTwoXCI's github");
          return;
        }
        std::string path = "/tinfoil/zerotwoxci/ncas/";
        std::string fullPath = "@Sdcard:/" + path;

        //Push a blank view ready for installation
        /*auto view = std::make_unique<tin::ui::ConsoleView>(3);
        manager.PushView(std::move(view));*/

        try
        {

            nx::fs::IFileSystem fileSystem;
            ASSERT_OK(fileSystem.OpenSdFileSystem(), "Failed to open SD file system");
            tin::install::nsp::SimpleFileSystem simpleFS(fileSystem, path, fullPath);
            tin::install::nsp::NSPInstallTask task(simpleFS, m_destStorageId, m_ignoreReqFirmVersion);

            task.Prepare();
            task.Begin();
        }
        catch (std::exception& e)
        {
            printf("Failed to install XCI!\n");
            LOG_DEBUG("Failed to install XCI");
            LOG_DEBUG("%s", e.what());
            fprintf(stdout, "%s", e.what());
        }
        int fin = reinitdir();
        if (fin == 1) {
          printf("done\n");
        }
        printf("Done!\n\nPress (B) to return.\n");
    }
}
